@@ -2,7 +2,6 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
-import asyncio
 import builtins
 import time
 from dataclasses import dataclass, field
@@ -12,14 +11,14 @@ from typing import Any, Awaitable, Callable, Dict, List, Optional, Tuple, Union
 from prometheus_client import Counter, Summary, start_http_server
 
 from volley.concurrency import run_async, run_worker_function
-from volley.config import load_yaml
+from volley.config import QueueConfig, load_yaml
 from volley.data_models import QueueMessage
 from volley.logging import logger
 from volley.models.base import message_model_handler
 from volley.profiles import ConnectionType, Profile, construct_profiles
 from volley.queues import DLQNotConfiguredError, Queue, construct_queue_map
 from volley.transport import DeliveryReport, delivery_success, produce_handler
-from volley.util import GracefulKiller
+from volley.util import FuncEnvelope, GracefulKiller
 
 # enables mocking the infinite loop to finite
 RUN_ONCE = False
@@ -44,7 +43,7 @@ class Engine:
             Not required if the application does not produce anywhere!
         dead_letter_queue: Points to the queue in configuration used as the dead letter queue.
         poll_interval_seconds: globally set time to to halt between polls on the consumer
-        queue_config: dictionary provided all queue configurations.
+        queue_config: either a list of QueueConfig (one for each queue) or a dictionary of queue configurations.
             Must provide one of queue_config or yaml_config_path
         yaml_config_path: path to a yaml config file.
             Must provide one of yaml_config_path or queue_config
@@ -62,7 +61,7 @@ class Engine:
 
     app_name: str = "volley"
     dead_letter_queue: Optional[str] = None
-    queue_config: Optional[Dict[str, Any]] = None
+    queue_config: Union[List[QueueConfig], Dict[str, Any], None] = None
     yaml_config_path: str = "./volley_config.yml"
     metrics_port: Optional[int] = 3000
     poll_interval_seconds: float = 1.0
@@ -78,8 +77,14 @@ class Engine:
         if self.output_queues == []:
             logger.warning("No output queues provided")
 
-        # if user provided config, use it
-        if self.queue_config:
+        # queue config can come in three ways
+        # 1. List[QueueConfig] , list of typed configurations, one for each queue
+        # 2. dictionary, queueName: configurations
+        # 3. yaml file
+
+        if isinstance(self.queue_config, list):
+            cfg = {x.name: x.to_dict() for x in self.queue_config}
+        elif isinstance(self.queue_config, dict):
             cfg = self.queue_config
         else:
             logger.info("loading configuration from %s", self.yaml_config_path)
@@ -90,13 +95,13 @@ class Engine:
             # if provided by user, DLQ becomes a producer target
             # flag the queue using the DLQ profile
             if self.dead_letter_queue not in cfg:
-                raise KeyError("%s not present in configuration", self.dead_letter_queue)
+                raise KeyError(f"{self.dead_letter_queue} not present in configuration")
             else:
                 self.output_queues.append(self.dead_letter_queue)
         else:
             logger.warning("DLQ not provided. Application will crash on schema violations")
 
-        # cfg can contain more queues that the app needs
+        # cfg can contain configs for more queues that the app needs
         # filter out queues that are not required by app
         cfg = {k: v for k, v in cfg.items() if k in [self.input_queue] + self.output_queues}
 
@@ -119,13 +124,11 @@ class Engine:
 
     def stream_app(  # noqa: C901
         self,
-        func: Callable[
-            [Any], Union[Awaitable[Any], List[Tuple[str, Any]], List[Tuple[str, Any, Dict[str, Any]]], bool]
-        ],
+        func: Callable[..., Union[Awaitable[Any], List[Tuple[str, Any]], List[Tuple[str, Any, Dict[str, Any]]], bool]],
     ) -> Callable[..., None]:
         """Main decorator for applications"""
 
-        is_coroutine = asyncio.iscoroutinefunction(func)
+        _func = FuncEnvelope(func)
 
         @run_async
         @wraps(func)
@@ -192,7 +195,12 @@ class Engine:
                     # this is happy path
                     # if outputs have been assigned it means this message is destined for a DLQ
                     _start_main = time.time()
-                    outputs = await run_worker_function(func, message=data_model, is_coroutine=is_coroutine)
+                    outputs = await run_worker_function(
+                        app_name=self.app_name,
+                        f=_func,
+                        message=data_model,
+                        ctx=in_message.message_context,
+                    )
                     _fun_duration = time.time() - _start_main
                     PROCESS_TIME.labels(volley_app=self.app_name, process_name="component").observe(_fun_duration)
 
@@ -230,7 +238,6 @@ class Engine:
                     break
 
             self.shutdown()
-            logger.info("Shutdown %s complete", self.app_name)
 
         # used for unit testing as a means to access the wrapped component without the decorator
         run_component.__wrapped__ = func  # type: ignore
@@ -246,3 +253,4 @@ class Engine:
             out_queue.producer_con.shutdown()
             logger.info("%s, %s shutdown complete", self.app_name, q_name)
         self.queue_map[self.input_queue].consumer_con.shutdown()
+        logger.info("Shutdown %s complete", self.app_name)
